const _ = require('lodash');
const Joi = require('lx-pjoi');
const evalModel = require('./evalModel');
const getModelDefinition = require('./getModelDefinition');


function mapInclude(include) {
  return (typeof include === 'Array') ? include.map(item => ({
    model: evalModel(item.model),
    as: item.as,
    include: mapInclude(item.include),
  })) : [];
}

function mapOffset(page, limit) {
  return (page - 1) * limit;
}

function mapOrder(order) {
  if (!order) {
    return;
  }
  if (typeof order === 'string') {
    order = [order];
  }
  return order.map(entry => {
    if (typeof entry === 'string') {
      if (entry.slice(0, 1) === '-') {
        return [entry.slice(1), 'DESC'];
      }
    }
    return entry;
  });
}

function extractModelIndexedFields(definition) {
  const defined = definition[2].indexes ? definition[2].indexes.map(index => index.fields) : [];
  const generated = Object.keys(_.pickBy(
    definition[1],
    options => options.primaryKey || options.unique
  ));
  const flatten = _.flattenDeep(defined.concat(generated));
  const unique = _.uniq(flatten);
  return unique.map(_.camelCase);
}

function allowDescOrder(uniqueIndexes) {
  return uniqueIndexes.concat(uniqueIndexes.map(index => `-${index}`));
}

function buildWhere(keys) {
  const where = _.zipObject(keys, keys.map(key => Joi.any())); 
  const reserved = {
    $and: Joi.object().keys(where),
    $or: Joi.array().items(Joi.object().keys(where)),
    $gt: Joi.any(),
    $lt: Joi.any(),
    $gte: Joi.any(),
    $lte: Joi.any(),
    $ne: Joi.any(),
    $between: Joi.array(),
    $notBetween: Joi.array(),
    $in: Joi.array(),
    $notIn: Joi.array(),
    $like: Joi.string(),
    $notLike: Joi.string(),
    $iLike: Joi.string(),
    $notILike: Joi.string(),
    $not: Joi.boolean(),
  };
  Object.assign(where, reserved);
  return Joi.object().keys(where);
}

function makeList({ model, queryDefaultLimit }) {
  const listModels = function *(args) {
    args = args || {};
    model = evalModel(model);
    const { limit, page, where, order, include } = yield Joi.validate(args, listModels.schema, { allowUnknown: true, abortEarly: false });
    const sqlParams = {
      limit,
      include: mapInclude(include) || undefined,
      offset: mapOffset(page, limit),
      where,
      order: mapOrder(order),
    };
    if (include) {
      sqlParams.include = mapInclude(include);
    }
    const { count, rows } = yield model.findAndCountAll(sqlParams);
    return {
      meta: {
        page: count > 0 ? page : 0,
        hits: rows.length,
        totalHits: count,
        totalPages: Math.ceil(count / limit),
      },
      result: rows,
    };
  };

  const definition = getModelDefinition(model);
  const indexedFields = extractModelIndexedFields(definition);
  const orderedFields = allowDescOrder(indexedFields);
  listModels.schema = Joi.object().keys({
    include: Joi.array(),
    limit: Joi.number().integer().min(1).max(100).default(queryDefaultLimit),
    page: Joi.number().integer().min(1).default(1),
    where: buildWhere(indexedFields),
    include: Joi.any(),
    order: Joi.alternatives().try(
      Joi.string().valid(orderedFields),
      Joi.array().items(Joi.string().valid(orderedFields))
    ),
  });

  return listModels;
}

module.exports = makeList;

