const Joi = require('lx-pjoi');
const fetch = require('isomorphic-fetch');
const format = require('es6-template-strings');
const { client } = require('lx-models');
const { NotAuthorizedException, NotFoundException } = require('lx-errors');
const config = require('../../config');
const session = require('../session');
const { servicesApiUrl, awsAcccessKey, awsSecretKey } = config;


const createContactSellerEmailTmpl = ({ fullName, senderEmail, itemUrl, itemTitle, message }) => `
Hi ${fullName},

You have received an enquiry for "${itemTitle}" from "${senderEmail}".

You can visit the item at "${itemUrl}" and follow up by replying to "${senderEmail}".


Message content:
-----------------------------------------

${message}

-----------------------------------------


Luxify Team

Hong Kong
Level 9, Core C, Cyberport 3
100 Cyberport Road
Hong Kong
Email: concierge@luxify.com
`;


function *contactSeller(args) {
  const { User, Item } = client.getModels();
  const { id, email, message, itemId } = yield Joi.validate(args, contactSeller.schema);

  // checking and authenticating
  const user = session.get('user');
  const contextEmail = user ? user.email : email;
  if (!contextEmail) {
    throw new NotAuthorizedException();
  }
  const seller = yield User.findOne({
    where: {
      id,
      isSuspended: false,
    }
  });
  if (!seller) {
    throw new NotFoundException();
  }
  const item = yield Item.findOne({
    where: {
      id: itemId,
      isSuspended: false,
    }
  });

  // preparing template
  const body = createContactSellerEmailTmpl({
    fullName: seller.fullName,
    senderEmail: contextEmail,
    itemTitle: item.title,
    itemUrl: `${config.host}/items/${item.id}`,
    message,
  });
  const emailOptions = {
    accessKey: config.awsAcccessKey,
    secretKey: config.awsSecretKey,
    region: config.awsSesRegion,
    sender: config.awsSesSender,
    subject: `Luxify - New Message for "${item.title}"`,
    type: 'Text',
    body,
    to: [seller.email],
    cc: [contextEmail],
    bcc: [config.awsSesSender],
  }

  // avoid sending to seller on non-production environment
  if (process.env.NODE_ENV !== 'production') {
    emailOptions.to = [config.awsSesSender];
  }

  // send the email
  const result = yield fetch(`${config.servicesApiUrl}/send-email`, {
    method: 'POST',
    body: JSON.stringify(emailOptions),
  });
  const json = yield result.json();
  if (json.ok) {
    return {
      result: json.result,
    };
  }
  throw new Error(json);
}

contactSeller.schema = Joi.object().keys({
  email: Joi.string().email().optional(),
  message: Joi.string().required(),
  itemId: Joi.number().required(),
});


module.exports = contactSeller;

