const { ValidationErrorException, NotAuthorizedException } = require('lx-errors');
const session = require('../session');
const Joi = require('lx-pjoi');
const { client } = require('lx-models');


function *createUser(args) {
  const User = client.getModels().User;
  const input = yield Joi.validate(args, createUser.schema);
  const currentUser = session.get('user');

  // non-admin user can only sign up, creating user with default role 'user' only
  if (!currentUser || currentUser.roleName !== 'admin') {
    if (input.roleName) {
      if (input.roleName !== 'user') {
        throw NotAuthorizedException();
      }
    }
  }

  // default username to be same as email, as it is indeed never used.
  if (!input.username) {
    input.username = input.email;
  }

  // proceed to creating the user
  if (yield User.findOne({ where: { username: input.username }, raw: true })) {
    throw new ValidationErrorException('Username has already been used.');
  }
  if (yield User.findOne({ where: { email: input.email }, raw: true })) {
    throw new ValidationErrorException('Email has already been used.');
  }
  if (yield User.findOne({ where: { companyRegNumber: input.companyRegNumber }, raw: true })) {
    throw new ValidationErrorException('Company registration number exists already.');
  }

  const result = yield User.create(input, { raw: true });
  return {
    result,
  };
}


createUser.schema = Joi.object().keys({
  sourceId: Joi.string().trim().max(255, 'utf8'),
  sourceName: Joi.string().trim().max(255, 'utf8').lowercase(),
  isSuspended: Joi.boolean(),
  roleName: Joi.string().trim().lowercase().default('user'),
  username: Joi.string().trim().min(4).max(127).lowercase().optional(),
  email: Joi.string().trim().lowercase().min(5).max(127).email().required(),
  companyAddress: Joi.string().trim(),
  companyRegNumber: Joi.string().trim(),
  companyLogoUrl: Joi.string().trim().uri(),
  companyName: Joi.string().trim(),
  companySummary: Joi.string().trim(),
  contactDetails: Joi.string().trim(),
  countryCode: Joi.string().trim().uppercase(),
  currencyCode: Joi.string().trim().uppercase(),
  firstName: Joi.string().trim().max(50, 'utf8'),
  lastName: Joi.string().trim().max(50, 'utf8'),
  languageCode: Joi.string().trim().lowercase().default('eng'),
  password: Joi.string().min(6),
  phoneNumbers: Joi.array().items(Joi.string()),
  socialFacebookUrl: Joi.string().uri(),
  socialInstagramUlr: Joi.string().uri(),
  socialPinterestUrl: Joi.string().uri(),
  socialTwitterUrl: Joi.string().uri(),
  websiteUrl: Joi.string().uri(),
}).with('sourceId', 'sourceName');

module.exports = createUser;

